# -*- coding: shift_jis -*-
# Rsygh1LB
#
# Copyright (c) 2006,2007 Takehiro Ihara.
# Copyright (c) 2006,2007 Satoshi Fukutomi.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHORS AND CONTRIBUTORS ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
#
#
#
package Trend::Article;

require v5.8.0;
use warnings;
use strict;
use Encode;
use IO::File;

use Trend::Feed;
use Trend::Source;
use Trend::Sort;
use Trend::Select;
use Trend::Config;

#
# RXgN^B
#
sub new($$) {
    my $self = shift(@_);
    my $keyword = shift(@_);
    return bless({keyword => $keyword,
                  sources => []});
}

#
# L[[hƂRSSƂɃ\[X擾B
#
sub getSources($) {
    my $self = shift(@_);
    my $encoded_keyword = encode('euc-jp', $self->{keyword});
    $encoded_keyword =~ s/(\W)/'%'.unpack('H2', $1)/eg;
    my $uri = sprintf($Trend::Config::keyworddiary_tmpl, $encoded_keyword);
    my $feed = Trend::Feed->new($uri);
    $feed->download();
    my @links = $feed->getLinks();
    my @paragraphs = ();
    foreach my $link (@links) {
        my $source = Trend::Source->new($link);
        push(@{$self->{sources}}, $source);
        $source->download();
        $source->parseHTML();
        $source->selectSections($self->{keyword});
        push(@paragraphs, $source->getParagraphs());
    }
    $self->{paragraphs} = \@paragraphs;
    return @paragraphs;
}

#
# ꗗ̃_vB
# foOpB
#
sub dump($$) {
    my $self = shift(@_);
    my $summary = $self->{candidateSummary};
    my @time = localtime();
    my $output = IO::File->new(sprintf('%s/%d-%02d-%02d',
                                       $Trend::Config::dump_dir,
                                       1900+$time[5], 1+$time[4], $time[3]),
                               'a');

    $output->print(encode('utf8', sprintf("* %s\n", $self->{keyword})));
    # 1̌ɂāAXRA̍̂7B
    my %prev = ();
    foreach my $sentence (@{$summary->[0]}) {
        if (! $prev{$sentence->{string}}) {
            $output->print(encode('utf8', $sentence->{string}), "\n");
            $prev{$sentence->{string}} = 1;
            if (keys(%prev) >= 7) {
                last;
            }
        }
    }
    $output->print("--------\n");

    # 1`7̌₩烉_7B
    my @all = (@{$summary->[0]}, @{$summary->[2]});
    for (my $i=$#all; $i>=0; $i--) {
        my $j = int(rand(0+@all));
        ($all[$i], $all[$j]) = ($all[$j], $all[$i]);
    }
    %prev = ();
    foreach my $sentence (@all) {
        if (! $prev{$sentence->{string}}) {
            $output->print(encode('utf8', $sentence->{string}), "\n");
            $prev{$sentence->{string}} = 1;
            if (keys(%prev) >= 7) {
                last;
            }
        }
    }
    $output->print("--------\n\n");
    $output->close();
}

#
# ̊ŕIсAъB
# v̊ě𐶐B
#
sub selectSentences($) {
    my $self = shift(@_);
    my $sort = Trend::Sort->new();
    my $sentence;
    my $sentences;
    my $summary = [];   # v̊ěB

    # 1A7B
    # L[[h܂ޕB
    $sentences = $sort->sort([@{$self->{paragraphs}}],
                    keywords => [$self->{keyword}],
                    penalty => 0.4);
    push(@{$summary}, $sentences);
    my $secondKeywords = $sort->relatedWords(7);

    # 2A7B
    # L[[h܂ޕA
    # L[[hEL[[hpoP̃XRAႭݒ肷B
    $sentences = $sort->sort([@{$self->{paragraphs}}],
                    keywords => [$self->{keyword}],
                    except => [$self->{keyword}],
                    penalty => 0.3);
    push(@{$summary}, $sentences);

    # 3A5B
    # 1ƂقړAL[[h͑2gB
    $sentences = $sort->sort([@{$self->{paragraphs}}],
                    keywords => $secondKeywords,
                    penalty => 0.35);
    push(@{$summary}, $sentences);

    # 4A6B
    # 2ƂقړAL[[h͑2gB
    $sentences = $sort->sort([@{$self->{paragraphs}}],
                    keywords => $secondKeywords,
                    except => $secondKeywords,
                    penalty => 0.3);
    push(@{$summary}, $sentences);

    $self->{candidateSummary} = $summary;
    $self->{secondKeywords} = $secondKeywords;
    $self->dump();
    return $summary;
}

#
# v̐B
#
sub summary($) {
    my $self = shift(@_);
    my $candidate = $self->{candidateSummary};
    my $select = Trend::Select->new();

    # 1B
    # ̂AƂXRA̍B
    $select->push($candidate->[0]);

    # 2B
    # ̂AƂXRA̍B
    # 1Ǝ߂ĂȂƁB
    $select->push($candidate->[1], threshold => 0.9); 

    # 3B
    # 1A2(臒l)łĂ镶B
    # 1A2Ƌʂ2܂ł邱ƁB
    # Y镶ȂꍇB
    my $commonWords = $select->usedWords($self->{secondKeywords});
    my $relatedSentences = $select->similaritySort($candidate->[2]);
    my $keySentences = $select->checkWords($relatedSentences, $commonWords);
    my $last = $select->push($keySentences, threshold=>0.8);

    # 4B
    # ȑO̕Ǝ߂ĂȂƁB
    # 3Ŏg2܂ł邱ƁB
    my $usedCommonWords = $select->usedWords($commonWords, target=>$last);
    $keySentences = $select->checkWords($candidate->[3], $usedCommonWords);
    $select->push($keySentences, threshold=>0.8);

    # 5B
    # 3ƓlB
    # 3Ɋ܂܂2͎gȂB
    my $unnsedCommonWords = $select->unusedWords($commonWords, target=>$last);
    $relatedSentences = $select->similaritySort($candidate->[2]);
    $keySentences = $select->checkWords($relatedSentences, $unnsedCommonWords);
    $select->push($keySentences, threshold=>0.8);

    # 6B
    # ȑO̕Ǝ߂ĂȂƁB
    # 5Ŏg2܂ł邱ƁB
    $usedCommonWords = $select->usedWords($commonWords, target=>$last);
    $keySentences = $select->checkWords($candidate->[3], $usedCommonWords);
    $select->push($keySentences, threshold=>0.8);

    # 7B
    # `e(`e܂)A莌̑o܂ޕB
    # z𓾂̂ړIB
    # ȑO̕Ǝ߂ĂȂƁB
    # ƂZB
    $keySentences = $select->checkPOS([@{$candidate->[0]}, @{$candidate->[1]}],
                                      [$Trend::Config::pos_adjective,
                                       $Trend::Config::pos_determine]);
    $select->push($keySentences, threshold=>0.8);

    $self->{summary} = $select->{summary};
    return $self->{summary};
}

#
# HTML̃GXP[vB
#
sub escape($$;$) {
    my $self = shift(@_);
    my $string = shift(@_);
    my $quote = undef;
    if (@_) {
        $quote = shift(@_);
    }
    $string =~ s/&/&amp;/g;
    $string =~ s/</&lt;/g;
    $string =~ s/>/&gt;/g;
    if ($quote) {
        $string =~ s/"/&quote;/g;
    }
    return $string;
}

#
# v񂪓K؂ł邩ǂ̃`FbNB
#
# 2̏𓯎ɖv̂ݒʂB
#       E3ӏȏォ̈pł邱ƁB
#       E5ȏ܂ނƁB
#
sub check($) {
    my $self = shift(@_);
    my %uri = ();
    foreach my $sentence (@{$self->{summary}}) {
        $uri{$sentence->{uri}} = 1;
    }
    return (keys(%uri) >= 3) && (@{$self->{summary}} >= 5);
}

#
# `ƏóB
#
sub write($$) {
    my $self = shift(@_);
    my $output = shift(@_);
    my $buf = sprintf("* %s\n", $self->{keyword});
    foreach my $sentence (@{$self->{summary}}) {
        my $body = $sentence->{string};
        $body =~ s/$Trend::Config::spec_string//g;
        $body =~ s/^ *//;
        $buf .= sprintf("%s%s<a href=\"%s\">%s</a>\n",
                        $self->escape($body),
                        $Trend::Config::char_dot,
                        $self->escape($sentence->{uri}, 1),
                        $Trend::Config::char_tri);
    }
    $output->print(encode('utf8', $buf), "\n");
}

1;
