# -*- coding: shift_jis -*-
#
# vɎgIԁB
#
# Copyright (c) 2006,2007 Takehiro Ihara.
# Copyright (c) 2006,2007 Satoshi Fukutomi.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHORS AND CONTRIBUTORS ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
#
#
package Trend::Select;

require v5.8.0;
use warnings;
use strict;

#
# RXgN^B
#
sub new($) {
    my $self = shift(@_);
    return bless({summary => []});
}

#
# łɍ̗pꂽɑ΂ގxB
#
# SĂ̕ɂėގx𒲂ׂőlB
#
# 1ƕ2̗ގx = max(ގx1, ގx2)
# ގx1 = ʂĊ܂܂Pꐔ / 1̒Pꐔ
# ގx2 = ʂĊ܂܂Pꐔ / 2̒Pꐔ
#
sub similarity($$) {
    my $self = shift(@_);
    my $sentence = shift(@_);
    my $max = 0;
    if (! @{$sentence->{words}}) {
        return 1;
    }
    foreach my $prev (@{$self->{summary}}) {
        my $common = 0;
        foreach my $word1 (@{$sentence->{words}}) {
            foreach my $word2 (@{$prev->{words}}) {
                if ($word1->{appear} eq $word2->{appear}) {
                    $common += 1;
                }
            }
        }
        if (@{$prev->{words}}) {
            my $sim1 = $common / @{$sentence->{words}};
            my $sim2 = $common / @{$prev->{words}};
            if ($max < $sim1) {
                $max = $sim1;
            } elsif ($max < $sim2) {
                $max = $sim2;
            }
        }
    }
    return $max;
}

#
# łɍ̗pꂽƂ̗ގxɃ\[gB
#
sub similaritySort($$) {
    my $self = shift(@_);
    my $sentences = shift(@_);
    return [sort {$self->similarity($b) <=> $self->similarity($a)}
            @{$sentences}];
}

#
# ̒ǉB
# ̕ŏ猩ĂāA
# łɂ镶Ƃ܂莗ĂȂ݂ΒǉB
#
sub push($$%) {
    my $self = shift(@_);
    my $sentences = shift(@_);
    my %opt = @_;
    my $threshold = $opt{threshold} || 2;
    foreach my $sentence (@{$sentences}) {
        if ((! @{$self->{summary}}) ||
            ($self->similarity($sentence) < $threshold)) {
            push(@{$self->{summary}}, $sentence);
            return $sentence;
        }
    }
    return undef;
}

#
# Ɋ܂܂PB
#
sub usedWords($$%) {
    my $self = shift(@_);
    my $words = shift(@_);
    my %opt = @_;
    my $target = $self->{summary};
    if (exists $opt{target}) {
        if ($opt{target}) {
            $target = [$opt{target}];
        } else {
            return [];
        }
    }
    my $usedWords = [];
    foreach my $word (@{$words}) {
        foreach my $prev (@${target}) {
            if ($prev->{string} =~ /\Q$word\E/) {
                CORE::push(@{$usedWords}, $word);
                last;
            }
        }
    }
    return $usedWords;
}

#
# Ɋ܂܂ȂPB
#
sub unusedWords($$%) {
    my $self = shift(@_);
    my $words = shift(@_);
    my %opt = @_;
    my $target = $self->{summary};
    if (exists $opt{target}) {
        if ($opt{target}) {
            $target = [$opt{target}];
        } else {
            return $words;
        }
    }
    my $unusedWords = [];
    CHECK_WORD: foreach my $word (@{$words}) {
        foreach my $prev (@${target}) {
            if ($prev->{string} =~ /\Q$word\E/) {
                next CHECK_WORD;
            }
        }
        CORE::push(@{$unusedWords}, $word);
    }
    return $unusedWords;
}

#
# w肵L[[hԂB
#
sub checkWords($$$) {
    my $self = shift(@_);
    my $sentences = shift(@_);
    my $keywords = shift(@_);
    my $buf = [];
    foreach my $sentence (@{$sentences}) {
        foreach my $word (@{$keywords})  {
            if ($sentence->{string} =~ /\Q$word\E/) {
                CORE::push(@{$buf}, $sentence);
                last;
            }
        }
    }
    return $buf;
}

#
# w肵iSĊ܂ޕԂB
#
sub checkPOS($$$) {
    my $self = shift(@_);
    my $sentences = shift(@_);
    my $pos = shift(@_);
    my $buf = [];
    CHECK_POS: foreach my $sentence (@{$sentences}) {
        foreach my $pos (@{$pos}) {
            local $_;
            if (! grep $_->{pos} eq $pos, @{$sentence->{words}}) {
                next CHECK_POS;
            }
        }
        CORE::push(@{$buf}, $sentence);
    }
    return $buf;
}

1;
